/*********************************************************************
 *
 *              dsPIC30F4013 UART code
 *
 *********************************************************************
 * FileName:        main.c
 * Dependencies:    p30F4013.h
 * Date:            16.3.2006
 * Processor:       dsPIC30F4013
 * Compiler:        MPLAB C30
 *
 * Author               Date       Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Kubik                16.3.2006  Original        (Rev 1.0)
 *
 ********************************************************************/

#include <p30fxxxx.h>
#include <i2c.h>
#include <stdio.h>

#include "defs.h"
#include "sed1335.h"
#define Fcy  (14318180 / 2)                                //To allow calculation of baud rate for display

_FOSC(CSW_FSCM_OFF & HS2_PLL4);
_FWDT(WDT_OFF);

//---------------------------------------------------------------------------
//Prototypes


//---------------------------------------------------------------------------
//Defines
#define UART_BAUD 9600
#define TEMP_ADDRESS (0x9E >> 1)                       // I2C address of the TI TMP100

//---------------------------------------------------------------------------
//Variables
uint8_t i2c_buff[8];


//---------------------------------------------------------------------------
//UART init
void uart1_init(unsigned long baud)
{

    U1BRG = Fcy / (16L * baud) - 1;
    U1MODE = 0x8000;
    U1STA = 0x0400;
}

void uart2_init(unsigned long baud)
{

    U2BRG = Fcy / (16L * baud) - 1;
    U2MODE = 0x8000;
    U2STA = 0x0400;
}

void lcd_delay_us(uint16_t delay)
{

    while (delay-- > 0) {
        nop();
        nop();
        nop();
        nop();
        nop();
        nop();
        nop();
        nop();
        nop();
        nop();
    }
}

//---------------------------------------------------------------------------
// I2C - address is allways 7bit!
#define RTC_ADDRESS 0x6F

// #define debug(val) {if(_debug) printf(#val);}
// #define debug(val) {if(_debug) printf(#val);}
// #define debug(val) {}
// {if(_debug) lcd_delay_us(10);}
// int _debug = 0;

void i2c_send(uint8_t addr, int len, uint8_t *buffer) {

    IdleI2C();

    StartI2C();
    while(I2CCONbits.SEN);
    MasterWriteI2C(addr << 1);
    while(len-- > 0) {
        while(I2CSTATbits.TRSTAT);
        while(I2CSTATbits.ACKSTAT);
        MasterWriteI2C(*buffer++);
    }
    while(I2CSTATbits.TRSTAT);
    while(I2CSTATbits.ACKSTAT);
    StopI2C();
    while(I2CCONbits.PEN);
    IdleI2C();
}

int i2c_receive(uint8_t addr, int len, uint8_t *buffer) {

    IdleI2C();

    StartI2C();
    while(I2CCONbits.SEN);
    MasterWriteI2C((addr << 1) | 0x01);
    while(len-- > 0) {
        while(I2CSTATbits.TRSTAT);
        while(I2CSTATbits.ACKSTAT);
        *buffer++ = MasterReadI2C();
        if(len > 0) {
            AckI2C();
        } else {
            NotAckI2C();
        }

    }
	lcd_delay_us(100);
    while(I2CSTATbits.ACKSTAT);
    StopI2C();
    while(I2CCONbits.PEN);
    IdleI2C();
    return 0;
}

int i2c_detect(uint8_t addr) {
    int present;

    IdleI2C();

    StartI2C();
    while(I2CCONbits.SEN);
    MasterWriteI2C(addr << 1);
    while(I2CSTATbits.TRSTAT);
    present = !I2CSTATbits.ACKSTAT;
    StopI2C();
    while(I2CCONbits.PEN);
    IdleI2C();
    return present;
}


//-----------------------------------
// Temperature read routines
//-----------------------------------
uint8_t temp_read(uint8_t addr) {

    i2c_buff[0] = addr;
    i2c_send(TEMP_ADDRESS, 1, i2c_buff);
    i2c_receive(TEMP_ADDRESS, 1, i2c_buff);
	return i2c_buff[0];
}


//-----------------------------------
// RTC read and write routines
//-----------------------------------
void RTC_write(uint8_t addr, uint8_t val) {

    i2c_buff[0] = 0;
    i2c_buff[1] = addr;
    i2c_buff[2] = val;
    i2c_send(RTC_ADDRESS, 3, i2c_buff);
}

uint8_t RTC_read(uint8_t addr) {

    i2c_buff[0] = 0;
    i2c_buff[1] = addr;
    i2c_send(RTC_ADDRESS, 2, i2c_buff);
    i2c_receive(RTC_ADDRESS, 1, i2c_buff);
	return i2c_buff[0];
}

uint8_t RTC_read_bcd(uint8_t addr) {
	uint8_t c;

	c = RTC_read(addr);
	return (c & 0x0F) + (c >> 4) * 10;
}


//---------------------------------------------------------------------------
//write override (to redirect printf to UART2)
int write(int handle, void *buffer, unsigned int len)
{
    if(handle == 1) {
        lcd_putc(*((unsigned char *) buffer));
    } else if(handle == 2) {
        while (U2STAbits.UTXBF);                  // wait until transmit buffer empty
        U2TXREG =  *((unsigned char*)buffer);     // type casting and dereferencing  of the void pointer
    }
    return len;
}

// void uart_putc(uint8_t c) {
// 	while (U2STAbits.UTXBF);                  // wait until transmit buffer empty
// 	U2TXREG =  ((unsigned char)c);            // type casting and dereferencing  of the void pointer
// }
// 

#ifdef PICTURE
const uint8_t r1[] = {
#include "r1.h"
};
#endif

//---------------------------------------------------------------------------
//Main routine
int main(void)
{
    int i;
    uint8_t *ptr;

    lcd_delay_us(50000);
    port_init();


    lcd_init();
    lcd_clear();

    uart2_init(9600);
    fprintf(stderr, "Hello\r\n");

#ifdef PICTURE
    lcd_set_cursor(G_START);
    lcd_write_cmd(CMD_MWRITE);

    ptr = r1;
    for (i = 0; i < 128 * 16; i++) {
        lcd_write_data(*ptr++);
    }
#endif

//I2C_ON&I2C_IDLE_CON&I2C_CLK_HLD&I2C_IPMI_DIS&I2C_7BIT_ADD&I2C_SLW_DIS&I2C_SM_DIS&I2C_GCALL_DIS&I2C_STR_DIS&I2C_NACK&I2C_ACK_DIS&I2C_RCV_DIS&I2C_STOP_DIS&I2C_RESTART_DIS&I2C_START_DIS
    OpenI2C(0xA000, 129);

	char buff[32];

//#define RTC_ADDRESS 0xDE
	printf("I2C search: ");
    for(i = 0; i < 0x80; i++) {
//        printf("%d (%x)...", i, i);
        if(i2c_detect(i)) {
            printf("%2.2X ", i << 1);
        }
    }

	printf("\n\r");
    uint8_t sec, min, hr, day, mon, yr;
    sec = min = hr = 0;
    day = 11;
    mon = 5;
    yr = 6;

	if(RTC_read(0x3F) & 0x01) {	       // Total power failure, RTC needs to be restarted
		RTC_write(0x3F, 0x02);
		RTC_write(0x3F, 0x06);
	    RTC_write(0x30, ((sec / 10) << 4) | (sec % 10));
    	RTC_write(0x31, ((min / 10) << 4) | (min % 10));
	    RTC_write(0x32, ((hr  / 10) << 4) | (hr  % 10) | 0x80);
    	RTC_write(0x33, ((day / 10) << 4) | (day % 10));
	    RTC_write(0x34, ((mon / 10) << 4) | (mon % 10));
    	RTC_write(0x35, ((yr  / 10) << 4) | (yr  % 10));
	}

	uint8_t b;
	while(1) {
        b = RTC_read_bcd(0x30);                 // Check if time has changed
		if(b == sec) continue;
		sec = b;
        min = RTC_read_bcd(0x31);
        hr = RTC_read(0x32);
        hr = ((hr & 0x30) >> 4) * 10 + (hr & 0x0F);
//         day = RTC_read_bcd(0x33);
//         mon = RTC_read_bcd(0x34);
//         yr = RTC_read_bcd(0x35);

        b = temp_read(0);
//         printf("\r%2.2d.%2.2d %2.2d:%2.2d:%2.2d ", day, mon, hr, min, sec);
        printf("\r%2.2d:%2.2d:%2.2d %2.2X", hr, min, sec, b);
    }

    while (1) {}
}                                                          //End of main()
