#include <p30fxxxx.h>
#include "defs.h"
#include "port.h"
#include "sed1335.h"

static int _lcd_x, _lcd_y;

void lcd_write_cmd(uint8_t cmd)
{

    gpio_lo(1);                                            // Select the LCD
    nop();
    port_write(LCD_COMMAND, cmd);
    nop();
    gpio_hi(1);
    nop();
}

void lcd_write_data(uint8_t cmd)
{

    gpio_lo(1);                                            // Select the LCD
    nop();
    port_write(LCD_DATA, cmd);
    nop();
    gpio_hi(1);
    nop();
}

void lcd_init(void)
{

    gpio_out(1);                                           // Configure LCD /CS and set it to high
    gpio_hi(1);

    gpio_out(2);                                           // Configure LCD /RES and pulse it to reset the display
    gpio_lo(2);
    lcd_delay_us(5000);
    gpio_hi(2);
    lcd_delay_us(5000);

    lcd_write_cmd(0x58);                                   //Display Off
    lcd_write_data(0x56);

    lcd_write_cmd(0x40);                                   //System Set
    lcd_write_data(0x38);                                  //P1
    lcd_write_data(0x87);                                  //P2  Char X Size
    lcd_write_data(0x07);                                  //P3  Char Y Size
    lcd_write_data(0x0F);                                  //P4  X Size
    lcd_write_data(0x20);                                  //P5  X Scan Size
    lcd_write_data(0x7F);                                  //P6  Y Scan Size
    lcd_write_data(0x10);                                  //P7  X Memory Size Low
    lcd_write_data(0x00);                                  //P8  X Memory Size High

    lcd_write_cmd(0x44);                                   //Scroll
    lcd_write_data(0x00);                                  //Adr1L    Obere Hlfte TXT
    lcd_write_data(T_START >> 8);                          //Adr1H
    lcd_write_data(0x40);                                  //SL1
    lcd_write_data(0x00);                                  //Adr2L    Obere Hlfte Grafik
    lcd_write_data(G_START >> 8);                          //Adr2H
    lcd_write_data(0x40);                                  //SL2
    lcd_write_data(0x80);                                  //Adr3L    Untere Hlfte TXT
    lcd_write_data(T_START >> 8);                          //Adr3H
    lcd_write_data(0x00);                                  //Adr4L    Untere Hlfte Grafik
    lcd_write_data((G_START >> 8) + 4);                    //Adr4H

    lcd_write_cmd(0x4C);                                   //Cursor Direction

    lcd_write_cmd(0x5A);                                   //Horizontal Scroll
    lcd_write_data(0x00);

    lcd_write_cmd(0x5B);                                   //Overlay
    lcd_write_data(0x01);                                  //TXT xor Grafik

    lcd_write_cmd(0x5D);                                   //Cursor Form
    lcd_write_data(0x05);
    lcd_write_data(0x86);

    lcd_write_cmd(0x5C);                                   //CGRAM ADR
    lcd_write_data(0x00);
    lcd_write_data(30);

    lcd_write_cmd(0x59);                                   //Display On
    lcd_write_data(0x54);                                  //Enable all planes, cursor off

    lcd_write_cmd(0x46);                                   //Set Cursor
    lcd_write_data(0x00);
    lcd_write_data(0x00);

    _lcd_x = _lcd_y = 0;
}

void lcd_set_cursor(uint16_t addr)
{

    lcd_write_cmd(0x46);                                   //Set Cursor
    lcd_write_data(addr & 0xFF);
    lcd_write_data(addr >> 8);
}

void lcd_clear(void)
{
    int i;

    lcd_set_cursor(T_START);                               //setzte cursor an obere hlfte Dipslay Text Mod
    lcd_write_cmd(0x42);                                   //Schreibcomanndo
    for (i = 0; i < (16 * 16); i++) {
        lcd_write_data(' ');                               //Schreibe leerzeichen um display zu lschen
    }

    lcd_set_cursor(G_START);                               //setzte cursor auf layer 2 Grafik obere hlfte
    lcd_write_cmd(0x42);
    for (i = 0; i < (16 * 128); i++) {
        lcd_write_data(0x00);                              // leere dieplay mit leeren pixeln
    }
}

void lcd_gotoxy(int x, int y)
{

    if ((x >= 0) && (x < 16)) {
        _lcd_x = x;
    }

    if ((y >= 0) && (y < 16)) {
        _lcd_y = y;
    }
}


void lcd_putc(unsigned char c)
{

    if (c < 32) {                                          // Control characters
        switch (c) {
        case 13:                                          // CR
            _lcd_x = 0;
            break;
        case 10:                                          // LF - line feed is interpreted as CR/LF
            if (_lcd_y < 15) {  // (most programs expect that)
                _lcd_y++;
                _lcd_x = 0;                                 
            }
            break;
        case 12:                                          // FF - clears the screen
            lcd_clear();
            break;
        case 9:                                           // Tab - jumps at the next MOD 4 pos, if not possible, does nothing
            if (_lcd_x < 12) {
                _lcd_x = (_lcd_x + 4) & ~0x03;
            }
            break;
        case 8:     // BS - backspaces one character, does not go to previous line
            if(_lcd_x > 0) {
                _lcd_x--;
            }
            break;
        }
    } else {
        lcd_set_cursor(T_START + _lcd_y * 16 + _lcd_x);
        lcd_write_cmd(CMD_MWRITE);
        lcd_write_data(c);

        _lcd_x++;
        if (_lcd_x >= 16) {
            _lcd_x = 0;
            _lcd_y++;
            if (_lcd_y >= 16) {
                _lcd_y = 0;
            }
        }
    }
}
